/*
 * This file is the main starting point where you can import js,
 * css, sass and other files.
 *
 * Examples:
 *   import "./sass/main.scss";
 *   import "./js/cool_beans.js";
 */

console.log("Webpack is compiling my assets.");
